#include "Tbsmod.h"

int AD9789_Channel(struct FrqBndC *FBC, char enable)
{
	unsigned char buff[4] = { 0 };
	ad9789_rd_nBytes(1, 0x05, buff);

	if(0 == enable) {
		if(1 == FBC->channels) {
			buff[0] = buff[0] & 0x0E;
		}
		else if(2 == FBC->channels) {
			buff[0] = buff[0] & 0x0D;
		}
		else if(3 == FBC->channels) {
			buff[0] = buff[0] & 0x0B;
		}
		else if(4 == FBC->channels) {
			buff[0] = buff[0] & 0x07;
		}
		else {
			printf("Select channel 1-4\n");
			exit(1);
		}

	}
	else if(1 == enable) {
		if(1 == FBC->channels) {
			buff[0] = buff[0] | 0x01;
		}
		else if(2 == FBC->channels) {
			buff[0] = buff[0] | 0x02;
		}
		else if(3 == FBC->channels) {
			buff[0] = buff[0] | 0x04;
		}
		else if(4 == FBC->channels) {
			buff[0] = buff[0] | 0x08;
		}
		else {
			printf("Select channel 1-4\n");
			exit(1);
		}
	}
	else {
		printf("Enable the AD9789 channel(0:Disable 1:Enable)\n");
		exit(1);
	}

	ad9789_wt_nBytes(1, 0x05, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(1, 0x05, buff);
	printf("AD9789 read ChEnable: 0x%02X\n", buff[0]);
	return 0;
}

int AD9789_Gain(struct FrqBndC *FBC)
{
	unsigned char buff[4] = { 0 };
	unsigned char tmp = 0;
	buff[0] = FBC->gain;
	int addr = 0x25;
	//printf("FBC->gain =  %X\n", buff[0]);
	if(1 == FBC->channels) {
		addr = 0x25;
	}
	else if(2 == FBC->channels) {
		addr = 0x26;
	}
	else if(3 == FBC->channels) {
		addr = 0x27;
	}
	else if(4 == FBC->channels) {
		addr = 0x28;
	}
	else {
		printf("Select channel 1-4\n");
		exit(1);
	}

	tmp = buff[0];
	ad9789_wt_nBytes(1, addr, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(1, addr, buff);
	printf("AD9789 read Gain: 0x%02X\n", buff[0]);
	if(tmp != buff[0]) {
		printf("AD9789 communication error\n");
		return 1;
	}


	ad9789_rd_nBytes(1, 0x05, buff);
	if(1 == FBC->channels) {//enable
		buff[0] = buff[0] | 0x01;
	}
	else if(2 == FBC->channels) {
		buff[0] = buff[0] | 0x02;
	}
	else if(3 == FBC->channels) {
		buff[0] = buff[0] | 0x04;
	}
	else if(4 == FBC->channels) {
		buff[0] = buff[0] | 0x08;
	}
	ad9789_wt_nBytes(1, 0x05, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(1, 0x05, buff);
	printf("AD9789 read ChEnable: 0x%02X\n", buff[0]);
	return 0;
}

int AD9789_Frq(struct FrqBndC *FBC)
{
	unsigned long Frq0, Frq1, Frq2, Frq3;
	unsigned char buff[4] = { 0 };
	unsigned char tmp[4] = { 0 };
	unsigned char QAMRRC = 0xC0;

	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };
	unsigned char tmpdata1 = 0;
	unsigned char tmpdata2 = 0;
	unsigned char tmpdata3 = 0;
	unsigned char tmpdata4 = 0;
	unsigned char bandwidth;


	if(FBC->bandwidth == 6 || FBC->bandwidth == 7 || FBC->bandwidth == 8){
		;
	}
	else {
		printf("Select bandwidth 6 or 7 or 8\n");
		exit(1);
	}

	ad9789_rd_nBytes(3, 0x1B, buff);
	if(buff[0] == 0x42 && buff[1] == 0x72 && buff[2] == 0x00) {
		bandwidth = 6;
	}
	else if(buff[0] == 0x71 && buff[1] == 0xE8 && buff[2] == 0x00) {
		bandwidth = 7;
	}
	else if(buff[0] == 0x63 && buff[1] == 0xAB && buff[2] == 0x00) {
		bandwidth = 8;
	}


	if(FBC->bandwidth != bandwidth) {

		tmpreg[0] = 0x11;//通道1,选中[7-0]
		tmpreg[1] = 0x80 + 0x0C;

		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
		if((0xFF == tmpbuf[0] && 0xFF == tmpbuf[1] && 0xFF == tmpbuf[2] && 0xFF == tmpbuf[3]) ||\
		   (0x00 == tmpbuf[0] && 0x00 == tmpbuf[1] && 0x00 == tmpbuf[2] && 0x00 == tmpbuf[3])) {
			printf("IP core communication error\n");
			return 1;
		}

		//LayerA Reset 1
		tmpdata1 = tmpbuf[3];
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpdata1 | 0x01;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		tmpreg[0] = 0x21;//通道2
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 1
		tmpdata2 = tmpbuf[3];
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpdata2 | 0x01;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);


		tmpreg[0] = 0x41;//通道3
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 1
		tmpdata3 = tmpbuf[3];
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpdata3 | 0x01;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);


		tmpreg[0] = 0x81;//通道4
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 1
		tmpdata4 = tmpbuf[3];
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpdata4 | 0x01;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	//config center freq
	unsigned long CenterF;
	//CenterF = FBC->frequency + (FBC->bandwidth * 3 / 2);

	//printf("Frq: %ld  CenterF: %ld FBC->bandwidth : %d\n", FBC->frequency, CenterF, FBC->bandwidth);
	Frq0 = ((double)16777215 * FBC->frequency / (135 * 1000)) + 0.5;
	buff[2] = Frq0 & 0xFF;
	buff[1] = (Frq0 >> 8) & 0xFF;
	buff[0] = (Frq0 >> 16) & 0xFF;
	tmp[2] = buff[2];
	tmp[1] = buff[1];
	tmp[0] = buff[0];
	ad9789_wt_nBytes(3, 0x0C, buff);
	//printf("Frq0 %ld %X %X %X\n", Frq0, buff[0], buff[1], buff[2]);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x0C, buff);
	printf("Frq0 : %02X %02X %02X\n", buff[0], buff[1], buff[2]);
	if(tmp[2] != buff[2] || tmp[1] != buff[1] || tmp[0] != buff[0]) {
		printf("AD9789 communication error\n");
		return 1;
	}


	Frq1 = ((double)16777215 * (FBC->frequency + FBC->bandwidth * 1000) / (135 * 1000)) + 0.5;
	buff[2] = Frq1 & 0xFF;
	buff[1] = (Frq1 >> 8) & 0xFF;
	buff[0] = (Frq1 >> 16) & 0xFF;
	ad9789_wt_nBytes(3, 0x0F, buff);
	//printf("Frq1 %ld %X %X %X\n", Frq1, buff[0], buff[1], buff[2]);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x0F, buff);
	printf("Frq1 : %02X %02X %02X\n", buff[0], buff[1], buff[2]);

	Frq2 = ((double)16777215 * (FBC->frequency + FBC->bandwidth * 2 * 1000) / (135 * 1000)) + 0.5;
	buff[2] = Frq2 & 0xFF;
	buff[1] = (Frq2 >> 8) & 0xFF;
	buff[0] = (Frq2 >> 16) & 0xFF;
	ad9789_wt_nBytes(3, 0x12, buff);
	//printf("Frq2 %ld %X %X %X\n", Frq2, buff[0], buff[1], buff[2]);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x12, buff);
	printf("Frq2 : %02X %02X %02X\n", buff[0], buff[1], buff[2]);

	Frq3 = ((double)16777215 * (FBC->frequency + FBC->bandwidth * 3 * 1000) / (135 * 1000)) + 0.5;
	buff[2] = Frq3 & 0xFF;
	buff[1] = (Frq3 >> 8) & 0xFF;
	buff[0] = (Frq3 >> 16) & 0xFF;
	ad9789_wt_nBytes(3, 0x15, buff);
	//printf("Frq3 %ld %X %X %X\n", Frq3, buff[0], buff[1], buff[2]);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x15, buff);
	printf("Frq3 : %02X %02X %02X\n", buff[0], buff[1], buff[2]);
	//Q
	buff[2] = 0x00;
	buff[1] = 0x00;
	buff[0] = 0x80;
	ad9789_wt_nBytes(3, 0x18, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x18, buff);
	printf("Q : %02X %02X %02X\n", buff[0], buff[1], buff[2]);

	//P
	if(FBC->bandwidth == 6) {
		QAMRRC = 0xC0;
		buff[2] = 0x00;
		buff[1] = 0x72;
		buff[0] = 0x42;
	}
	else if(FBC->bandwidth == 7) {
		QAMRRC = 0xD0;
		buff[2] = 0x00;
		buff[1] = 0xE8;
		buff[0] = 0x71;
	}
	else if(FBC->bandwidth == 8) {
		QAMRRC = 0xD0;
		buff[2] = 0x00;
		buff[1] = 0xAB;
		buff[0] = 0x63;
	}
	ad9789_wt_nBytes(3, 0x1B, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(3, 0x1B, buff);
	printf("P : %02X %02X %02X\n", buff[0], buff[1], buff[2]);

	//CenterF = (CenterF * 65536) / 2160;
	CenterF = ((double)65535 * (FBC->frequency + FBC->bandwidth * 3 * 500) / (2160 * 1000)) + 0.5;
	buff[1] = CenterF & 0xFF;
	buff[0] = (CenterF >> 8) & 0xFF; 
	ad9789_wt_nBytes(2, 0x1D, buff); 
	//printf("CenterF %ld %X %X\n", CenterF, buff[0], buff[1]);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(2, 0x1D, buff);
	printf("CenterF : %02X %02X\n", buff[0], buff[1]);

	buff[0] = QAMRRC;
	ad9789_wt_nBytes(1, 0x06, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(2, 0x06, buff);
	printf("QAM & RRC filter : %02X\n", buff[0]);


	//Gain
	buff[0] = FBC->gain;
	int addr = 0x25;
	ad9789_wt_nBytes(1, addr, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(1, addr, buff);
	printf("AD9789 read Gain: 0x%02X\n", buff[0]);

	//open channel 1
	ad9789_rd_nBytes(1, 0x05, buff);
	buff[0] = buff[0] | 0x01;
	ad9789_wt_nBytes(1, 0x05, buff);
	memset(buff,0,sizeof(buff));
	ad9789_rd_nBytes(1, 0x05, buff);
	printf("AD9789 read ChEnable: 0x%02X\n\n", buff[0]);

	//update
	buff[0] = 0x80;
	ad9789_wt_nBytes(1, 0x1E, buff);
	free(FBC);


	if(FBC->bandwidth != bandwidth) {
		usleep(100000);
		//LayerA Reset 0
		tmpreg[0] = 0x11;//通道1,选中[7-0]
		tmpbuf[3] = tmpdata1;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 0
		tmpreg[0] = 0x21;//通道1,选中[7-0]
		tmpbuf[3] = tmpdata2;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 0
		tmpreg[0] = 0x41;//通道1,选中[7-0]
		tmpbuf[3] = tmpdata3;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

		//LayerA Reset 0
		tmpreg[0] = 0x81;//通道1,选中[7-0]
		tmpbuf[3] = tmpdata4;
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
		printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	return 0;
}

int AD9789_Configuration_Init(char *AD9789Init)
{
	FILE *fp1 = NULL;
	char str[512] = { 0 };
	int add, data;
	unsigned char buf[2] = { 0 };
	int i,Rmu = 0;

	fp1 = fopen(AD9789Init,"r");
	if(fp1 == NULL)
	{
		printf("fopen %s fail\n", AD9789Init);
		exit(1);	
	}

	while(!feof(fp1))
	{
		memset(str,0,sizeof(str));
		fgets(str,sizeof(str),fp1);
		if(str[0] == '\n' || str[0] == '\r' || str[0] == '\0' || 0 == strncmp(str,"#", 1)) {
			continue;
		}
		else {
			//printf("%s",str);
			sscanf(str, "%X %X", &add, &data);
			buf[0] = data;
			if(add == 0x04) {
				Rmu++;
				if(Rmu > 1) {
					buf[0] = 0x00;//0x04 read mu clock
					for (i = 0; i < 100; i++){
						ad9789_rd_nBytes(1, 0x04, buf);
						if (buf[0] == 0x08)
							break;
						usleep(10000);
					}
					if (buf[0] != 0x08) {
						printf("MU delay controller locking failed\n");
						return 1;
					}
					continue;
				}
			}
			//printf("0x%02X 0x%02X\n", add, buf[0]);
			ad9789_wt_nBytes(1, add, buf);
			//buf[0] = 0x00;
			//ad9789_rd_nBytes(1, add, buf);
			//printf("AD9789 read velue: %x\n", buf[0]);
		}
	}
	fclose(fp1);

	return 0;
}

char *trim(char* str)
{
	if (str == NULL || *str == '\0')
	{
		return str;
	}
	char *start = str;

	int i=0;
	while (isspace(*start) && *start)
	{
		++start;
		++i;
	}
	int len = strlen(str);
	char *end = str + len -1;

	while (*end && isspace(*end)){
		*end = '\0';
		--end;
	}
	memmove(str, start, strlen(str) - i + 1);
	return str;
}

double Layer_Bitrate(char *Layer, int BW, int GuardIntervalV, int QAM, double CoderateV, int Segment) 
{
	double FL = 0, LayerBitrate = 0, TotalBitrate = 0;
	FL = 204 * BW * (1 + (1 / pow(2 , 5 - GuardIntervalV)));
	LayerBitrate = (24 * QAM * CoderateV * 188000000 * 8.0) / FL;
	TotalBitrate = LayerBitrate * Segment;
	//printf("FL = %f LayerBitrate = %.2f TotalBitrate = %.2f\n", FL, LayerBitrate, TotalBitrate);
	printf("%s\t:\t%.2fbps\n", Layer, TotalBitrate);
	return TotalBitrate;
}

//复位ISDBT IP核
void ResetCore()
{
	unsigned char buf[2] = { 0 };
	buf[0] = 1;
	tbs_write(0x38, *(u32*)&buf[0]);
	usleep(100000);
	buf[0] = 0;
	tbs_write(0x38, *(u32*)&buf[0]);
	usleep(100000);

}

void ResetISDBT_Register(char channel) 
{
	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };
	unsigned char tmpdata[1] = { 0 };

	if(1 == channel) {
		tmpreg[0] = 0x11;//通道1,选中[31-0]
	}
	else if(2 == channel) {
		tmpreg[0] = 0x21;//通道2
	}
	else if(3 == channel) {
		tmpreg[0] = 0x41;//通道3
	}
	else if(4 == channel) {
		tmpreg[0] = 0x81;//通道4
	}
	else {
		printf("Select Channel 1-4\n");
		exit(1);
	}

	tmpreg[1] = 0x80 + 0x0C;

	//printf("tmpreg[0]=%02X tmpreg[1]=%02X\n", tmpreg[0], tmpreg[1]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	//printf("I2C Read: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	//LayerA Reset 1
	tmpdata[0] = tmpbuf[3];
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = tmpdata[0] | 0x01;
	i2c_write(0x46,tmpreg,4,tmpbuf);
	//printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	//printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	//memset(tmpbuf,0,sizeof(tmpbuf));
	//i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	//printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	usleep(100000);
	//LayerA Reset 0
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = tmpdata[0];
	i2c_write(0x46,tmpreg,4,tmpbuf);
	//printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	//printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	//memset(tmpbuf,0,sizeof(tmpbuf));
	//i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	//printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

}

//在字符串的某个位置插入一个字符
int insert(char str[],char ch,char num)
{
	int i;
	if(num > strlen(str))
	{
		printf("The position you entered is beyond the length of the string!"); 
		return -1;
	}
	else
	{
		for(i = strlen(str) ; i >= num ; i--)
		{
			str[i+1] = str[i];
		}
		str[i+1] = str[i];
		str[i] = ch;
		//printf("%d %c %c\n",i,str[i+1],str[i]);
		return 0;
	}
}

//char to hex
int chartohex(char str[], unsigned char buff[])
{
	int i = 0, j = 0, st;
	unsigned char tmpbuf = 0x00;

	if(str[0] == '0' && (str[1]  == 'x' || str[1] == 'X')) {//0x
		st = 2;
		if(0 != strlen(str) % 2)//单
		{
			insert(str,'0',3);
		}
	}
	else {
		st = 0;
		if(0 != (strlen(str) % 2))
		{
			insert(str,'0',1);
		}
	}

	for(j = st ; j < strlen(str) ; j++) {
		if(0 == j % 2) {
			if(str[j] >= 'A' && str[j] <= 'F') {
				tmpbuf = tmpbuf + ((str[j] - 55) << 4);
			}
			else if(str[j] >= 'a' && str[j] <= 'f') {
				tmpbuf = tmpbuf + ((str[j] - 87) << 4);
			}
			else if(str[j] >= '0' && str[j] <= '9') {
				tmpbuf = tmpbuf + ((str[j] - '0') << 4);
			}
			else {
				return -1;
			}
			//printf("tmpbuf1[%d]=%02X\n\n",i,tmpbuf);
		}
		else {
				if(str[j] >= 'A' && str[j] <= 'F') {
					tmpbuf = tmpbuf + (str[j] - 55);
				}
				else if(str[j] >= 'a' && str[j] <= 'f') {
					tmpbuf = tmpbuf + (str[j] - 87);
				}
				else if(str[j] >= '0' && str[j] <= '9') {
					tmpbuf = tmpbuf + (str[j] - '0');
				}
				else {
					return -1;
				}
			//printf("tmpbuf0[%d]=%02X\n\n",i,tmpbuf);
			buff[i] = tmpbuf;
			tmpbuf = 0x00;
			i++;
		}
	}
	return i;
}

void ReadOneFull(char channel, char *ReadAdd) 
{
	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };

	unsigned char buff[4] = { 0 };
	int mark = chartohex(ReadAdd, buff);
	if(-1 == mark) {
		printf("Please enter the correct register address or value\n");
		exit(1);
	}

	if(0x0C == buff[0]) { ; }
	else {
		printf("Please enter the correct register address!\n");
		exit(1);
	}

	if(1 == channel) {
		tmpreg[0] = 0x1F;//通道1,选中[31-0]
	}
	else if(2 == channel) {
		tmpreg[0] = 0x2F;//通道2
	}
	else if(3 == channel) {
		tmpreg[0] = 0x4F;//通道3
	}
	else if(4 == channel) {
		tmpreg[0] = 0x8F;//通道4
	}
	else {
		printf("Select Channel 1-4\n");
		exit(1);
	}

	tmpreg[1] = 0x80 + buff[0];

	//printf("tmpreg[0]=%02X tmpreg[1]=%02X\n", tmpreg[0], tmpreg[1]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	//printf("Register: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	if((0xFF == tmpbuf[0] && 0xFF == tmpbuf[1] && 0xFF == tmpbuf[2] && 0xFF == tmpbuf[3]) ||\
	   (0x00 == tmpbuf[0] && 0x00 == tmpbuf[1] && 0x00 == tmpbuf[2] && 0x00 == tmpbuf[3])) {
		printf("IP core communication error\n");
		return;
	}

	u8 PartialRcp = (tmpbuf[2] & 0x10) >> 4;
	if(0 == PartialRcp) {
		printf("Full-Seg\n");
	}
	else if(1 == PartialRcp) {
		printf("One-Seg\n");
	}

}


void ReadISDBT_Register(char channel, char *ReadAdd) 
{
	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };

	unsigned char buff[4] = { 0 };
	int mark = chartohex(ReadAdd, buff);
	if(-1 == mark) {
		printf("Please enter the correct register address or value\n");
		exit(1);
	}

	if(0x00 == buff[0] || 0x04 == buff[0] ||  0x08 == buff[0] ||\
	   0x0C == buff[0] || 0x10 == buff[0] ||  0x14 == buff[0] ||\
	   0x18 == buff[0] || 0x1C == buff[0]) { ; }
	else {
		printf("Please enter the correct register address!\n");
		exit(1);
	}
	/*int j = 0;
	for(j = 0 ; j < mark ; j++) {
		printf("buff[%d]=%02X ",j,buff[j]);
	}
	printf("\n");*/

	if(1 == channel) {
		tmpreg[0] = 0x1F;//通道1,选中[31-0]
	}
	else if(2 == channel) {
		tmpreg[0] = 0x2F;//通道2
	}
	else if(3 == channel) {
		tmpreg[0] = 0x4F;//通道3
	}
	else if(4 == channel) {
		tmpreg[0] = 0x8F;//通道4
	}
	else {
		printf("Select Channel 1-4\n");
		exit(1);
	}

	tmpreg[1] = 0x80 + buff[0];

	//printf("tmpreg[0]=%02X tmpreg[1]=%02X\n", tmpreg[0], tmpreg[1]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	if(buff[0] == 0x04 || buff[0] == 0x10 || buff[0] == 0x18) {
		if(0xFF == tmpbuf[0] && 0xFF == tmpbuf[1] && 0xFF == tmpbuf[2] && 0xFF == tmpbuf[3]) {
			printf("IP core communication error\n");
			return;
		}
	}
	else {
		if((0xFF == tmpbuf[0] && 0xFF == tmpbuf[1] && 0xFF == tmpbuf[2] && 0xFF == tmpbuf[3]) ||\
		   (0x00 == tmpbuf[0] && 0x00 == tmpbuf[1] && 0x00 == tmpbuf[2] && 0x00 == tmpbuf[3])) {
			printf("IP core communication error\n");
			return;
		}
	}

	if(buff[0] == 0x00) {
		printf("Version register: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}
	//printf("Register: %02X %02X %02X %02X\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	if(0x04 == buff[0] || 0x10 == buff[0] || 0x18 == buff[0]) {
		u8 SyncStatus = tmpbuf[0] >> 5;
		u8 Stream = (tmpbuf[0] & 0x10) >> 4;
		unsigned int Bitrate = ((tmpbuf[0] & 0x0F) << 24) + (tmpbuf[1] << 16 ) + (tmpbuf[2] << 8) + tmpbuf[3];

		if(0x04 == buff[0])
			printf("\nstatus register Layer A:\n\n");
		else if(0x10 == buff[0])
			printf("\nstatus register Layer B:\n\n");
		else if(0x18 == buff[0])
			printf("\nstatus register Layer C:\n\n");

		if(0 == SyncStatus) {
			printf("Frame : Not synchronized\n");
		}
		else if(1 == SyncStatus) {
			printf("Frame : 188 bytes\n");
		}
		else if(2 == SyncStatus) {
			printf("Frame : 204 bytes\n");
		}
		else if(4 == SyncStatus) {
			printf("Frame : 208 bytes\n");
		}
		else if(7 == SyncStatus) {
			printf("Frame : Synchronized at non standard length frame\n");
		}

		if(0 == Stream) {
			printf("Stream : NO\n");
		}
		else if(1 == Stream) {
			printf("Stream : Input\n");
		}

		printf("Input Stream(Bitrate) : %d\n\n",Bitrate);
	}
	else if(0x08 == buff[0]) {
		u8 LostFrame = tmpbuf[0];
		u8 DeleteNullPacket = (tmpbuf[1] & 0x20) >> 5;
		u8 BypassPCR = (tmpbuf[1] & 0x10) >> 4;
		u8 SPIEdgeInput = (tmpbuf[1] & 0x08) >> 3;
		unsigned int FPGAClock = ((tmpbuf[1] & 0x07) << 16) + (tmpbuf[2] << 8) + tmpbuf[3];

		printf("\nconfiguration register Layer A:\n\n");
		printf("Lost Frame (max 255) : %d\n", LostFrame);

		if(0 == DeleteNullPacket) {
			printf("Null Packets : Keep\n");
		}
		else if(1 == DeleteNullPacket) {
			printf("Null Packets : Delete\n");
		}

		if(0 == BypassPCR) {
			printf("PCR Correction : ON\n");
		}
		else if(1 == BypassPCR) {
			printf("PCR Correction : OFF\n");
		}

		if(0 == SPIEdgeInput) {
			printf("Data Sampled : Falling Edge\n");
		}
		else if(1 == SPIEdgeInput) {
			printf("Data Sampled : Rising Edge\n");
		}

		printf("Clock frequency : %dKHz\n\n",FPGAClock);
	}
	else if(0x0C == buff[0]) {
		u8 Segment = tmpbuf[0] >> 4;
		u8 TimeInterleaver = tmpbuf[0] & 0x03;
		u8 FEC = (tmpbuf[1] & 0x70) >> 4;
		u8 Modulation = (tmpbuf[1] & 0x03);
		u8 GuardInterval = (tmpbuf[2] & 0x03);
		u8 PartialRcp = (tmpbuf[2] & 0x10) >> 4;
		u8 Mode = (tmpbuf[3] & 0x30)  >> 4;
		u8 Reset = tmpbuf[3] & 0x01;

		printf("\nModulator configuration register:\n\n");
		if(0 == Mode) {
			printf("iFFT : 2K\n");
		}
		else if(1 == Mode) {
			printf("iFFT : 4K\n");
		}
		else if(2 == Mode) {
			printf("iFFT : 8K\n");
		}

		if(0 == GuardInterval) {
			printf("GuardInterval : 1/32\n");
		}
		else if(1 == GuardInterval) {
			printf("GuardInterval : 1/16\n");
		}
		else if(2 == GuardInterval) {
			printf("GuardInterval : 1/8\n");
		}
		else if(3 == GuardInterval) {
			printf("GuardInterval : 1/4\n");
		}

		if(0 == PartialRcp) {
			printf("Partial Recpetion : Full-Seg\n");
		}
		else if(1 == PartialRcp) {
			printf("Partial Recpetion : One-Seg\n");
		}

		printf("Layer : A\n");
		if(0 == Modulation) {
			printf("Constellation : DQPSK\n");
		}
		else if(1 == Modulation) {
			printf("Constellation : QPSK\n");
		}
		else if(2 == Modulation) {
			printf("Constellation : 16QAM\n");
		}
		else if(3 == Modulation) {
			printf("Constellation : 64QAM\n");
		}

		if(0 == FEC) {
			printf("Coderate : 1/2\n");
		}
		else if(1 == FEC) {
			printf("Coderate : 2/3\n");
		}
		else if(2 == FEC) {
			printf("Coderate : 3/4\n");
		}
		else if(3 == FEC) {
			printf("Coderate : 5/6\n");
		}
		else if(4 == FEC) {
			printf("Coderate : 7/8\n");
		}
		else if(5 == FEC || 6 == FEC) {
			printf("Coderate : Reserved\n");
		}
		else if(7 == FEC) {
			printf("Coderate : Unused hierarchical layer\n");
		}

		printf("TimeInterleaver : %d\n",TimeInterleaver);

		if(0x00 == Segment || 0x0E == Segment) {
			printf("segments : Reserved\n");
		}
		else if(0x0F == Segment) {
			printf("segments : Unused hierarchical layer\n");
		}
		else {
			printf("segments : %d\n",Segment);
		}

		if(0 == Reset) {
			printf("core is running until a 1 is written on it\n\n");
		}
		else if(1 == Reset) {
			printf("core is reset until a 0 is written on it\n\n");
		}

	}
	else if(0x14 == buff[0] || 0x1C == buff[0]) {
		u8 Segment = tmpbuf[0] >> 4;	
		u8 TimeInterleaver = tmpbuf[0] & 0x03;
		u8 FEC = (tmpbuf[1] & 0x70) >> 4;
		u8 Modulation = (tmpbuf[1] & 0x03);
		u8 DeleteNullPacket = (tmpbuf[2] & 0x04) >> 2;
		u8 BypassPCR = (tmpbuf[2] & 0x02) >> 1;
		u8 SPIEdgeInput = tmpbuf[2] & 0x01;
		u8 LostFrame = tmpbuf[3];

		printf("\n");
		if(0x14 == buff[0]) {
			printf("\nLayer B configuration register:\n\n");
		}
		else if(0x1C == buff[0]) {
			printf("\nLayer C configuration register:\n\n");
		}

		if(0 == Modulation) {
			printf("Constellation : DQPSK\n");
		}
		else if(1 == Modulation) {
			printf("Constellation : QPSK\n");
		}
		else if(2 == Modulation) {
			printf("Constellation : 16QAM\n");
		}
		else if(3 == Modulation) {
			printf("Constellation : 64QAM\n");
		}

		if(0 == FEC) {
			printf("Coderate : 1/2\n");
		}
		else if(1 == FEC) {
			printf("Coderate : 2/3\n");
		}
		else if(2 == FEC) {
			printf("Coderate : 3/4\n");
		}
		else if(3 == FEC) {
			printf("Coderate : 5/6\n");
		}
		else if(4 == FEC) {
			printf("Coderate : 7/8\n");
		}
		else if(5 == FEC || 6 == FEC) {
			printf("Coderate : Reserved\n");
		}
		else if(7 == FEC) {
			printf("Coderate : Unused hierarchical layer\n");
		}

		printf("TimeInterleaver : %d\n",TimeInterleaver);

		if(0x00 == Segment || 0x0E == Segment) {
			printf("segments : Reserved\n\n");
		}
		else if(0x0F == Segment) {
			printf("segments : Unused hierarchical layer\n\n");
		}
		else {
			printf("segments : %d\n\n",Segment);
		}


		printf("Lost Frame(max 255) : %d\n", LostFrame);

		if(0 == DeleteNullPacket) {
			printf("Null Packets : Keep\n");
		}
		else if(1 == DeleteNullPacket) {
			printf("Null Packets : Delete\n");
		}


		if(0 == BypassPCR) {
			printf("PCR Correction : ON\n");
		}
		else if(1 == BypassPCR) {
			printf("PCR Correction : OFF\n");
		}

		if(0 == SPIEdgeInput) {
			printf("Data Sampled : Falling Edge\n\n");
		}
		else if(1 == SPIEdgeInput) {
			printf("Data Sampled : Rising Edge\n\n");
		}
	}
}

void delChar(char *str)
{
    int i, j = 0, mark = 0;
    int len = strlen(str);
 
    for(i = 0; i < len; i++)
    {
        if(str[i] == '\"')
        {
			mark++;
			continue;
        }
		if(mark == 1) {
			str[j] = str[i];
			j++;
		}
    }
	str[j] = '\0';
}
  

void ISDBT_Configuration(char *ISDBTC)
{
	FILE *fp1 = NULL;
	char str[512] = { 0 };
	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };
	unsigned long data[3] = { 0 };
	
	char *tmp0 = NULL, *tmp1 = NULL;
	char *tmp_str0 = NULL,*tmp_str1 = NULL;
	
	char LayerM = 0;
	char SegmentTotal = 0;
	int tmpdata = 0;

	int BW = 0;
	int GuardIntervalV = 0;
	int QAMV = 0;
	int QAMA = 0;
	int QAMB = 0;
	int QAMC = 0;
	double CoderateV = 0;
	double CoderateA = 0;
	double CoderateB = 0;
	double CoderateC = 0;
	int SegmentA = 0;
	int SegmentB = 0;
	int SegmentC = 0;
	double LayerABitrate = 0;
	double LayerBBitrate = 0;
	double LayerCBitrate = 0;

	fp1 = fopen(ISDBTC,"r");
	if(fp1 == NULL)
	{
		printf("fopen %s error\n", ISDBTC);
		exit(1);	
	}

	char value[100] = { 0 };
	while(!feof(fp1))
	{
		memset(str,0,sizeof(str));
		fgets(str,sizeof(str),fp1);
		if(str[0] == '\n' || str[0] == '\r' || str[0] == '\0' || 0 == strncmp(str,"#", 1) ||\
		   0 == strncmp(str,"{", 1) || 0 == strncmp(str,"}", 1)) {
			continue;
		}
		else {

			tmp0 = strtok(str, ":");
			tmp1 = strtok(NULL, ":");

			if(tmp0 == NULL || tmp1 == NULL)
			{	
				//printf("Please write the configuration file in the correct format\n");
				continue;
			}

			tmp_str0 = trim(tmp0);
			tmp_str1 = trim(tmp1);
			strcpy(value,tmp_str1);
			delChar(value);
			//printf("tmp_str0=%s value=%s\n\n", tmp_str0, value);
			if(0 == strncmp(tmp_str0, "\"Channel\"", 9)) {
				
				if(5 == strlen(value) && value[4] == '0') {
					tmpreg[0] = 0x1F;//通道1,选中[31-0]
				}
				else if(5 == strlen(value) && value[4] == '1') {
					tmpreg[0] = 0x2F;//通道2
				}
				else if(5 == strlen(value) && value[4] == '2') {
					tmpreg[0] = 0x4F;//通道3
				}
				else if(5 == strlen(value) && value[4] == '3') {
					tmpreg[0] = 0x8F;//通道4
				}
				else {
					printf("Select Channel1 to Channel4\n");
					exit(1);
				}
				//printf("strlen(value)=%d %x\n", strlen(value),tmpreg[0]);
			}
			else if(0 == strncmp(tmp_str0, "\"Bandwidth\"", 11)) {

				if(0 == strcmp(value, "6")) {
					BW = 252;
				}
				else if(0 == strcmp(value, "7")) {
					BW = 216;
				}
				else if(0 == strcmp(value, "8")) {
					BW = 189;
				}
				else {
					printf("Bandwidth only 6M or 7M or 8M\n");
					exit(1);
				}
				//printf("BW=%d\n", BW);
			}
			else if(0 == strncmp(tmp_str0, "\"Layer\"", 7)) {
				if(0 == strcmp(value, "A")) {
					LayerM = 1;
					//tmpreg[1] = 0x80 + 0x0C;
				}
				else if(0 == strcmp(value, "B")) {
					LayerM = 2;
					//tmpreg[1] = 0x80 + 0x14;
				}
				else if(0 == strcmp(value, "C")) {
					LayerM = 3;
					//tmpreg[1] = 0x80 + 0x1C;
				}
				else {
					printf("Layer only A or B or C\n");
					exit(1);
				}
				//printf("LayerM=%d\n", LayerM);
			}
			else if(0 == strncmp(tmp_str0, "\"iFFT\"", 6)) {
				if(0 == strcmp(value, "2K")) {
					tmpdata = 0;
				}
				else if(0 == strcmp(value, "4K")) {
					tmpdata = 1;
				}
				else if(0 == strcmp(value, "8K")) {
					tmpdata = 2;
				}
				else {
					printf("iFFT only 2K or 4K or 8K\n");
					exit(1);
				}
				data[0] = data[0] +  (tmpdata << 4);
				//data[0] = data[0] +  (4 << tmpdata);
			}
			else if(0 == strncmp(tmp_str0, "\"GuardInterval\"", 15)) {
				if(0 == strcmp(value, "1/32")) {
					tmpdata = 0;
					GuardIntervalV = 0;
				}
				else if(0 == strcmp(value, "1/16")) {
					tmpdata = 1;
					GuardIntervalV = 1;
				}
				else if(0 == strcmp(value, "1/8")) {
					tmpdata = 2;
					GuardIntervalV = 2;
				}
				else if(0 == strcmp(value, "1/4")) {
					tmpdata = 3;
					GuardIntervalV = 3;
				}
				else {
					printf("GuardInterval only 1/32 or 1/16 or 1/8 or 1/4\n");
					exit(1);
				}
				data[0] = data[0] +  (tmpdata << 8);
			}
			else if(0 == strncmp(tmp_str0, "\"PartialReception\"", 18)) {
				tmpdata = atoi(value);
				data[0] = data[0] + (tmpdata << 12);
			}
			else if(0 == strncmp(tmp_str0, "\"Constellation\"", 15)) {
				if(0 == strcmp(value, "DQPSK")) {
					tmpdata = 0;
					QAMV = 1;
				}
				else if(0 == strcmp(value, "QPSK")) {
					tmpdata = 1;
					QAMV = 1;
				}
				else if(0 == strcmp(value, "16QAM")) {
					tmpdata = 2;
					QAMV = 2;
				}
				else if(0 == strcmp(value, "64QAM")) {
					tmpdata = 3;
					QAMV = 3;
				}
				else {
					printf("Constellation only DQPSK or QPSK or 16QAM or 64QAM\n");
					exit(1);
				}

				if(1 == LayerM) {
					data[0] = data[0] +  (tmpdata << 16);
					QAMA = QAMV;
				}
				else if(2 == LayerM) {
					data[1] = data[1] +  (tmpdata << 16);
					QAMB = QAMV;
				}
				else if(3 == LayerM) {
					data[2] = data[2] +  (tmpdata << 16);
					QAMC = QAMV;
				}
			}
			else if(0 == strncmp(tmp_str0, "\"CodeRate\"", 10)) {
				if(0 == strcmp(value, "1/2")) {
					tmpdata = 0;
					CoderateV = (double)1/2;
				}
				else if(0 == strcmp(value, "2/3")) {
					tmpdata = 1;
					CoderateV = (double)2/3;
				}
				else if(0 == strcmp(value, "3/4")) {
					tmpdata = 2;
					CoderateV = (double)3/4;
				}
				else if(0 == strcmp(value, "5/6")) {
					tmpdata = 3;
					CoderateV = (double)5/6;
				}
				else if(0 == strcmp(value, "7/8")) {
					tmpdata = 4;
					CoderateV = (double)7/8;
				}
				else if(0 == strcmp(value, "Unused")) {
					tmpdata = 7;
					CoderateV = 0;
				}
				else {
					printf("Coderate only 1/2 or 2/3 or 3/4 or 5/6 or 7/8\n");
					exit(1);
				}

				if(1 == LayerM) {
					data[0] = data[0] +  (tmpdata << 20);
					CoderateA = CoderateV;
				}
				else if(2 == LayerM) {
					data[1] = data[1] +  (tmpdata << 20);
					CoderateB = CoderateV;
				}
				else if(3 == LayerM) {
					data[2] = data[2] +  (tmpdata << 20);
					CoderateC = CoderateV;
				}
			}
			else if(0 == strncmp(tmp_str0, "\"TimeInterleave\"", 16)) {
				tmpdata = atoi(value);

				if(0 == tmpdata || 1 == tmpdata || 2 == tmpdata || 3 == tmpdata) {
					;
				}
				else {
					printf("TimeInterleaver only 0 or 1 or 2 or 3\n");
					exit(1);
				}

				if(1 == LayerM) {
					data[0] = data[0] +  (tmpdata << 24);
				}
				else if(2 == LayerM) {
					data[1] = data[1] +  (tmpdata << 24);
				}
				else if(3 == LayerM) {
					data[2] = data[2] +  (tmpdata << 24);
				}
			}
			else if(0 == strncmp(tmp_str0, "\"Segment\"", 9)) {
				tmpdata = atoi(value);
				//printf("tmpdata=%d\n", tmpdata);
				if(1 == LayerM) {
					if(tmpdata == 0) {
						data[0] = data[0] + 0xF0000000;
					}
					else {
						data[0] = data[0] + (tmpdata << 28);
					}
					SegmentA =tmpdata;
				}
				else if(2 == LayerM) {
					if(tmpdata == 0) {
						data[1] = data[1] + 0xF0000000;
					}
					else {
						data[1] = data[1] + (tmpdata << 28);
					}
					SegmentB =tmpdata;
				}
				else if(3 == LayerM) {
					if(tmpdata == 0) {
						data[2] = data[2] + 0xF0000000;
					}
					else {
						data[2] = data[2] + (tmpdata << 28);
					}
					SegmentC =tmpdata;
				}
				SegmentTotal = SegmentTotal + tmpdata;
			}

		}

	}
	data[1] = data[1] +  (0 << 10);//0:keep 1:delete
	data[1] = data[1] +  (1 << 9);//0:on 1:off
	data[2] = data[2] +  (0 << 10);//0:keep 1:delete
	data[2] = data[2] +  (1 << 9);//0:on 1:off

	if(SegmentTotal <= 0 || SegmentTotal > 13) {
		printf("Segment<1-13> LayerA + LayerB + LayerC\n");
		exit(1);
	}
	//printf("data[0]=0x%04lX data[1]=0x%04lX data[2]=0x%04lX\n\n",data[0] ,data[1],data[2]);

	LayerABitrate = Layer_Bitrate("LayerA Bitrate", BW, GuardIntervalV, QAMA, CoderateA, SegmentA);
	LayerBBitrate = Layer_Bitrate("LayerB Bitrate",BW, GuardIntervalV, QAMB, CoderateB, SegmentB);
	LayerCBitrate = Layer_Bitrate("LayerC Bitrate",BW, GuardIntervalV, QAMC, CoderateC, SegmentC);
	printf("Tota Bitrate\t:\t%.2fbps\n\n", LayerABitrate + LayerBBitrate + LayerCBitrate);
	tmpreg[1] = 0x80 + 0x08;//LayerA
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = 0x58;
	tmpbuf[2] = 0x0F;
	tmpbuf[1] = 0x12;
	tmpbuf[0] = 0x00;//keep and off

	/*tmpbuf[3] = 0x58;
	tmpbuf[2] = 0x0F;
	tmpbuf[1] = 0x32;
	tmpbuf[0] = 0x00;//delete off*/

	/*tmpbuf[3] = 0x58;
	tmpbuf[2] = 0x0F;
	tmpbuf[1] = 0x22;
	tmpbuf[0] = 0x00;//delete and on*/

	/*tmpbuf[3] = 0x58;
	tmpbuf[2] = 0x0F;
	tmpbuf[1] = 0x02;
	tmpbuf[0] = 0x00;//keep and on*/

	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	if((0xFF == tmpbuf[0] && 0xFF == tmpbuf[1] && 0xFF == tmpbuf[2] && 0xFF == tmpbuf[3]) ||\
	   (0x00 == tmpbuf[0] && 0x00 == tmpbuf[1] && 0x00 == tmpbuf[2] && 0x00 == tmpbuf[3])) {
		printf("IP core communication error\n");
		return;
	}


	tmpreg[1] = 0x80 + 0x0C;//LayerA
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = data[0] & 0xFF;
	tmpbuf[2] = (data[0] >> 8) & 0xFF;
	tmpbuf[1] = (data[0] >> 16) & 0xFF;
	tmpbuf[0] = (data[0] >> 24) & 0XFF;
	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	tmpreg[1] = 0x80 + 0x14;//LayerB
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = data[1] & 0xFF;
	tmpbuf[2] = (data[1] >> 8) & 0xFF;
	tmpbuf[1] = (data[1] >> 16) & 0xFF;
	tmpbuf[0] = (data[1] >> 24) & 0XFF;
	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	tmpreg[1] = 0x80 + 0x1C;//LayerC
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = data[2] & 0xFF;
	tmpbuf[2] = (data[2] >> 8) & 0xFF;
	tmpbuf[1] = (data[2] >> 16) & 0xFF;
	tmpbuf[0] = (data[2] >> 24) & 0XFF;
	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	//LayerA Reset 1
	tmpreg[0] = tmpreg[0] & 0xF1;
	tmpreg[1] = 0x80 + 0x0C;
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = data[0] | 0x01;
	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	usleep(100000);
	//LayerA Reset 0
	memset(tmpbuf,0,sizeof(tmpbuf));
	tmpbuf[3] = data[0];
	i2c_write(0x46,tmpreg,4,tmpbuf);
	printf("PRT Add: %02X %02X\n", tmpreg[0], tmpreg[1]);
	printf("I2C Write: %02X %02X %02X %02X\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
	memset(tmpbuf,0,sizeof(tmpbuf));
	i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
	printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	fclose(fp1);
}

void PIDFilter_Configuration(char *PIDFilter)
{
	FILE *fp1 = NULL, *fp2 = NULL;
	char str[512] = { 0 };
	unsigned char tmpbuf[4] = { 0 };
	unsigned char tmpbufa[4] = { 0 };
	unsigned char tmpbufb[4] = { 0 };
	unsigned char tmpbufc[4] = { 0 };
	unsigned char tmpreg[4] = { 0 };

	char *tmp0 = NULL, *tmp1 = NULL, *tmp2 = NULL;
	char *tmp_str0 = NULL,*tmp_str1 = NULL;

	char channelbit = 16;
	char LayerAFilterNum = 0;
	char LayerBFilterNum = 0;
	char LayerCFilterNum = 0;

	char APIDOffset = 0x04;
	char BPIDOffset = 0x04;
	char CPIDOffset = 0x04;

	char LayerAM = 1;
	char LayerBM = 1;
	char LayerCM = 1;

	char LayerAS = 0;
	char LayerBS = 0;
	char LayerCS = 0;

	char WriteM = 0;

	fp1 = fopen(PIDFilter,"r");
	fp2 = fopen("/usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/TmpPIDFilter","wb");
	if(fp1 == NULL || fp2 == NULL)
	{
		printf("fopen PIDFilter error\n");
		exit(1);	
	}

	char value[100] = { 0 };
	char tmmchannel[100] = { 0 };
	char tmpvalue[100] = { 0 };
	printf("\n\n");
	while(!feof(fp1))
	{
		memset(str,0,sizeof(str));
		fgets(str,sizeof(str),fp1);
		tmp2 = trim(str);
		if(0 == strncmp(tmp2,"\"Channel\"",9) ||\
		   0 == strncmp(tmp2,"\"PID\"",5) ||\
		   0 == strncmp(tmp2,"\"Layer\"",7)) {

			//printf("%s\n",tmp2);

			tmp0 = strtok(str, ":");
			tmp1 = strtok(NULL, ":");

			tmp_str0 = trim(tmp0);
			tmp_str1 = trim(tmp1);
			strcpy(value,tmp_str1);
			delChar(value);
			
			if(0 == strncmp(tmp2,"\"Channel\"",9)) {
				strcpy(tmmchannel, "Channel");
				if(5 == strlen(value) && value[4] == '0') {
					strcat(tmmchannel, " 1\n");
				}
				else if(5 == strlen(value) && value[4] == '1') {
					strcat(tmmchannel, " 2\n");
				}
				else if(5 == strlen(value) && value[4] == '2') {
					strcat(tmmchannel, " 3\n");
				}
				else if(5 == strlen(value) && value[4] == '3') {
					strcat(tmmchannel, " 4\n");
				}
				else {
					printf("Select Channel1 to Channel4\n");
					exit(1);
				}
			}
			if(0 == strncmp(tmp2,"\"Layer\"",7)) {
				strcpy(tmpvalue, value);
			}
			if(0 == strncmp(tmp2,"\"PID\"",5)) {
				strcat(tmpvalue, " ");
				strcat(tmpvalue, value);
				strcat(tmpvalue, "\n");
				fputs(tmpvalue,fp2);
			}
			//printf("tmp_str0=%s value=%s\n", tmp_str0, value);
		}
	}
	fclose(fp2);

	fp2 = fopen("/usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/PIDFilter","wb");
	if(fp2 == NULL)
	{
		printf("fopen PIDFilter error\n");
		exit(1);	
	}
	fputs(tmmchannel,fp2);
	fclose(fp2);
	system("sort /usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/TmpPIDFilter >> /usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/PIDFilter");
	remove("/usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/TmpPIDFilter");


	fp1 = fopen("/usr/local/tbs/apache2.4.3/htdocs/www/cgi-bin/remux/TMCCP/PIDFilter","r");
	if(fp1 == NULL)
	{
		printf("fopen PIDFilter error\n");
		exit(1);	
	}
	while(!feof(fp1))
	{
		memset(str,0,sizeof(str));
		fgets(str,sizeof(str),fp1);
		if(str[0] == '\n' || str[0] == '\r' || str[0] == '\0' || 0 == strncmp(str,"#", 1)) {
			continue;
		}
		else {
			tmp0 = strtok(str, " ");
			tmp1 = strtok(NULL, " "); 
			
			if(tmp0 == NULL || tmp1 == NULL)
			{	
				printf("Please write the configuration file in the correct format\n");
				exit(1);
			}
			tmp_str0 = trim(tmp0);
			tmp_str1 = trim(tmp1);
			printf("tmp_str0=%s tmp_str1=%s\n", tmp_str0, tmp_str1);

			if(0 == strcmp(tmp_str0,"Channel")) {
				if(0 == strncmp(tmp_str1,"1", 1)) {
					channelbit = 1 << 4;
					continue;
				}
				else if(0 == strncmp(tmp_str1,"2", 1)) {
					channelbit = 1 << 5;
					continue;
				}
				else if(0 == strncmp(tmp_str1,"3", 1)) {
					channelbit = 1 << 6;
					continue;
				}
				else if(0 == strncmp(tmp_str1,"4", 1)) {
					channelbit = 1 << 7;
					continue;
				}
				else {
					printf("Select Channel1 to Channel4\n");
					exit(1);
				}
			}
			else if(0 == strcmp(tmp_str0, "A")) {
				tmpreg[0] = channelbit + (1 << 1);
				++LayerAFilterNum;

				if(1 == (LayerAFilterNum % 2)) {
					APIDOffset = 0x04 * LayerAM;//[12:0]
					LayerAM++;
					tmpbufa[1] = atoi(tmp_str1) & 0xFF;
					tmpbufa[0] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 0;
				}
				else {
					tmpbufa[3] = atoi(tmp_str1) & 0xFF;
					tmpbufa[2] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 1;//[28:16]
				}
				tmpreg[1] = APIDOffset;
				LayerAS = 1;
				LayerBS = 0;
				LayerCS = 0;
			}
			else if(0 == strcmp(tmp_str0, "B")) {
				if(0 == WriteM && 1 == LayerAS) {//A层单数 后面有B层
					memset(tmpbuf,0,sizeof(tmpbuf));
					tmpbuf[3] = tmpbufa[1];
					tmpbuf[2] = tmpbufa[0];
					i2c_write(0x46,tmpreg,4,tmpbuf);
					printf("1A tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
					memset(tmpbuf,0,sizeof(tmpbuf));
					i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
					printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

				}

				tmpreg[0] = channelbit + (1 << 2);
				++LayerBFilterNum;

				if(1 == (LayerBFilterNum % 2)) {
					BPIDOffset = 0x04 * LayerBM;//[12:0]
					LayerBM++;
					tmpbufb[1] = atoi(tmp_str1) & 0xFF;
					tmpbufb[0] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 0;
				}
				else {
					tmpbufb[3] = atoi(tmp_str1) & 0xFF;
					tmpbufb[2] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 1;//[28:16]
				}
				tmpreg[1] = BPIDOffset;
				LayerAS = 0;
				LayerBS = 1;
				LayerCS = 0;
			}
			else if(0 == strcmp(tmp_str0, "C")) {
				if(0 == WriteM && 1 == LayerBS) {//B层单数
					memset(tmpbuf,0,sizeof(tmpbuf));
					tmpbuf[3] = tmpbufb[1];
					tmpbuf[2] = tmpbufb[0];
					i2c_write(0x46,tmpreg,4,tmpbuf);
					printf("1B tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
					memset(tmpbuf,0,sizeof(tmpbuf));
					i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
					printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

				}

				tmpreg[0] = channelbit + (1 << 3);
				++LayerCFilterNum;

				if(1 == (LayerCFilterNum % 2)) {
					CPIDOffset = 0x04 * LayerCM;//[12:0]
					LayerCM++;
					tmpbufc[1] = atoi(tmp_str1) & 0xFF;
					tmpbufc[0] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 0;
				}
				else {
					tmpbufc[3] = atoi(tmp_str1) & 0xFF;
					tmpbufc[2] = (atoi(tmp_str1) >> 8) & 0xFF;
					WriteM = 1;//[28:16]
				}
				tmpreg[1] = CPIDOffset;
				LayerAS = 0;
				LayerBS = 0;
				LayerCS = 1;
			}
			else {
				continue;
			}
		}
		//printf("LayerFilterNum %d %d %d\n", LayerAFilterNum, LayerBFilterNum, LayerCFilterNum);
		//printf("tmpreg[0]=0x%02X tmpreg[1]=0x%02X\n", tmpreg[0], tmpreg[1]);
		//printf("tmpbuf[0]=0x%02X tmpbuf[1]=0x%02X tmpbuf[2]=0x%02X tmpbuf[3]=0x%02X\n\n", tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);

		//A层双数
		if(1 == LayerAS && 1 == WriteM) {
			tmpbuf[3] = tmpbufa[3];
			tmpbuf[2] = tmpbufa[2];
			tmpbuf[1] = tmpbufa[1];
			tmpbuf[0] = tmpbufa[0];
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("2A tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
		}
		
		//B层双数
		if(1 == LayerBS && 1 == WriteM) {
			tmpbuf[3] = tmpbufb[3];
			tmpbuf[2] = tmpbufb[2];
			tmpbuf[1] = tmpbufb[1];
			tmpbuf[0] = tmpbufb[0];
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("2B tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
		}

		//C层双数
		if(1 == LayerCS && 1 == WriteM) {
			tmpbuf[3] = tmpbufc[3];
			tmpbuf[2] = tmpbufc[2];
			tmpbuf[1] = tmpbufc[1];
			tmpbuf[0] = tmpbufc[0];
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("2C tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
		}
	}

	//A层单数 后面无B层
	if(1 == (LayerAFilterNum % 2) && 1 == LayerAS) {
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpbufa[1];
		tmpbuf[2] = tmpbufa[0];
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("1A tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	//B层单数 后面无C层
	if(1 == (LayerBFilterNum % 2) && 1 == LayerBS) {
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpbufb[1];
		tmpbuf[2] = tmpbufb[0];
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("1B tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	//C层单数
	if(1 == (LayerCFilterNum % 2)) {
		memset(tmpbuf,0,sizeof(tmpbuf));
		tmpbuf[3] = tmpbufc[1];
		tmpbuf[2] = tmpbufc[0];
		i2c_write(0x46,tmpreg,4,tmpbuf);
		printf("1C tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n", tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
		memset(tmpbuf,0,sizeof(tmpbuf));
		i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
		printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}
	
	unsigned long EnableBit = 0;
	printf("LayerFilterNum %d %d %d\n", LayerAFilterNum, LayerBFilterNum, LayerCFilterNum);
	
	if(LayerAFilterNum > 30 || LayerBFilterNum > 30 || LayerCFilterNum > 30) {
		printf("A maximum of 30 PIDs can be filtered per layer\n");
		exit(1);
	}

	//A TS-Enable
	if(LayerAFilterNum > 0) {
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 1);
			EnableBit = 0xFFFFFFFF >> (32 - LayerAFilterNum);
			tmpbuf[3] = EnableBit & 0xFF;
			tmpbuf[2] = (EnableBit >> 8) & 0xFF;
			tmpbuf[1] = (EnableBit >> 16) & 0xFF;
			tmpbuf[0] = (EnableBit >> 24) & 0xFF;
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("EnableBit=%lX tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n",EnableBit, tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	}
	else if(0 == LayerAFilterNum) {
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 1);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("tmpreg[0]=%02X %02X\n", tmpreg[0], tmpreg[1]);
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read LayerAFilterNum: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	//B TS-Enable
	if(LayerBFilterNum > 0) {
			EnableBit = 0;
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 2);
			EnableBit = 0xFFFFFFFF >> (32 - LayerBFilterNum);
			tmpbuf[3] = EnableBit & 0xFF;
			tmpbuf[2] = (EnableBit >> 8) & 0xFF;
			tmpbuf[1] = (EnableBit >> 16) & 0xFF;
			tmpbuf[0] = (EnableBit >> 24) & 0xFF;
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("EnableBit=%lX tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n",EnableBit, tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);

	}
	else if(0 == LayerBFilterNum) {
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 2);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("tmpreg[0]=%02X %02X\n", tmpreg[0], tmpreg[1]);
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read LayerBFilterNum: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

	//C TS-Enable
	if(LayerCFilterNum > 0) {
			EnableBit = 0;
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 3);
			EnableBit = 0xFFFFFFFF >> (32 - LayerCFilterNum);
			tmpbuf[3] = EnableBit & 0xFF;
			tmpbuf[2] = (EnableBit >> 8) & 0xFF;
			tmpbuf[1] = (EnableBit >> 16) & 0xFF;
			tmpbuf[0] = (EnableBit >> 24) & 0xFF;
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("EnableBit=%lX tmpreg[0]=%02X %02X tmpbuf[0]=%02X %02X %02X %02X\n",EnableBit, tmpreg[0], tmpreg[1], tmpbuf[0], tmpbuf[1], tmpbuf[2], tmpbuf[3]);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}
	else if(0 == LayerCFilterNum) {
			tmpreg[1] = 0x00;
			tmpreg[0] = channelbit + (1 << 3);
			memset(tmpbuf,0,sizeof(tmpbuf));
			i2c_write(0x46,tmpreg,4,tmpbuf);
			printf("tmpreg[0]=%02X %02X\n", tmpreg[0], tmpreg[1]);
			i2c_read_no_stop(0x46,tmpreg,2,tmpbuf,4);
			printf("I2C Read LayerCFilterNum: %02X %02X %02X %02X\n\n", tmpbuf[0],tmpbuf[1],tmpbuf[2],tmpbuf[3]);
	}

}
