#include "Tbsmod.h"

int i_frontend =-1;

static const struct option long_options[] =
{
{ "tbsmod",         required_argument, NULL, 'a' },
{ "mod", required_argument, NULL, 'b' },
{ "SPI selection", required_argument, NULL, 's' },
{ "AD9789 init", required_argument, NULL, 'i' },
{ "AD9789 channels", required_argument, NULL, 'c' },
{ "frequency",  required_argument, NULL, 'F' },
{ "bandwidth",  required_argument, NULL, 'B' },
{ "gain",  required_argument, NULL, 'G' },
{ "Enable ch",  required_argument, NULL, 'E' },
{ "ISDBT channel",  required_argument, NULL, 'C' },
{ "Reset ISDBT",  no_argument, NULL, 'r' },
{ "Reset IP Core",  no_argument, NULL, 'R' },
{ "read ISDBT",  required_argument, NULL, 'A' },
{ "ISDBTC",  required_argument, NULL, 'I' },
{ "PIDFilter",  required_argument, NULL, 'P' },
{ "OneFull",  no_argument, NULL, 'O' },
{ 0, 0, 0, 0 }
};

void usage()
{
    printf("\n");
    printf("Usage:\n");
    printf("-a : tbsmod\n");
    printf("-b : mod\n");
    printf("-s : SPI device selection(0: select AD9789 1:Upgrade FPGA)\n");
    printf("-i : AD9789 init\n");
    printf("-c : AD9789 channels (1-4)\n");
    printf("-F : frequency (100-866M)\n");
    printf("-B : bandwidth (6M 7M 8M)\n");
    printf("-G : gain (3-255)\n");
    printf("-E : Enable the AD9789 channel(0:Disable 1:Enable)\n");
    printf("-C : ISDBT channel (1-4)\n");
    printf("-r : Reset ISDBT register\n");
    printf("-R : Reset IP Core\n");
    printf("-A : Read ISDBT register\n");
    printf("-I : ISDBT configuration\n");
    printf("-P : PID Filter configuration\n");
    printf("eg : ./ISDBT -a 0 -b 0 -s 0 -i ./TMCCP/AD9789.conf \n");
    printf("   : ./ISDBT -a 0 -b 0 -F 474 -B 6 -G 128\n");
    printf("   : ./ISDBT -a 0 -b 0 -c 1 -G 128\n");
    printf("   : ./ISDBT -a 0 -b 0 -c 1 -E 0\n");
    printf("   : ./ISDBT -a 0 -b 0 -C 1 -r\n");
    printf("   : ./ISDBT -a 0 -b 0 -R\n");
    printf("   : ./ISDBT -a 0 -b 0 -C 1 -A 0x00\n");
    printf("   : ./ISDBT -a 0 -b 0 -I ./TMCCP/ISDBTI_1_0\n");
    printf("   : ./ISDBT -a 0 -b 0 -P ./TMCCP/PIDFilterI_1_0\n\n");
    printf("   :  0x00 Version\n");
    printf("   :  0x04 Layer A status\n");
    printf("   :  0x08 Layer A configuration\n");
    printf("   :  0x0C Modulator configuration\n");
    printf("   :  0x10 Layer B status\n");
    printf("   :  0x14 Layer B configuration\n");
    printf("   :  0x18 Layer C status\n");
    printf("   :  0x1C Layer C configuration\n");
    printf("\n");
    exit(1);
}

int main(int i_argc, char **pp_argv)
{ 		
    int c;
	char Adapter_EN = 0;
	char Fenum_EN = 0;
	char SpiS_EN = 0;
	char AD9789Init_EN = 0;
	char Channels_EN = 0;
	char Frequency_EN = 0;
	char Bandwidth_EN = 0;
	char Gain_EN = 0;
	char Enable_EN = 0;
	char ISDBTC_EN = 0;
	char PIDFilter_EN = 0;
	char ReadISDBT_EN = 0;
	char Channel_EN = 0;
	char ResetISDBTR_EN = 0;
	char ResetCore_EN = 0;
	char OneFull_EN = 0;

 	char spis = 0;
	unsigned long frequency = 474;
    unsigned char bandwidth = 6;
	unsigned char channels = 0;
	unsigned char gain = 128;
	char *ISDBTC = NULL;
	char *PIDFilter = NULL;
	char *AD9789Init = NULL;
	char *ReadAdd = NULL;
	char Channel = 0;
	char enable = 0;

	int adapter = 0;
	int fenum =0;

    if(i_argc == 1)
        usage();
    while ((c = getopt_long(i_argc, pp_argv, "a:b:s:c:F:B:G:E:i:C:rRA:I:P:O", long_options, NULL)) != -1)
    {
        switch(c)
        {
			case 'a':
				Adapter_EN = 1;
				adapter = strtol(optarg, NULL, 0);
				break;
			case 'b':
				Fenum_EN = 1;
				fenum = strtol(optarg, NULL, 0);
				break;
			case 's':
				SpiS_EN = 1;
				spis = strtol(optarg, NULL, 0);
				break;
				break;
			case 'c':
				Channels_EN = 1;
				channels = strtol(optarg, NULL, 0);
				break;
			case 'F':
				Frequency_EN = 1;
				frequency = atof(optarg) * 1000;
				break;
			case 'B':
				Bandwidth_EN = 1;
				bandwidth = strtol(optarg, NULL, 0);
				break;
			case 'G':
				Gain_EN = 1;
				gain = strtol(optarg, NULL, 0);
				break;
			case 'E':
				Enable_EN = 1;
				enable = strtol(optarg, NULL, 0);
				break;
			case 'i':
				AD9789Init_EN = 1;
				AD9789Init = optarg;
				break;
			case 'C':
				Channel_EN = 1;
				Channel = strtol(optarg, NULL, 0);
				break;
			case 'r':
				ResetISDBTR_EN = 1;
				break;
			case 'R':
				ResetCore_EN = 1;
				break;
			case 'A':
				ReadISDBT_EN = 1;
				ReadAdd = optarg;
				break;
			case 'I':
				ISDBTC_EN = 1;
				ISDBTC = optarg;
				break;
			case 'P':
				PIDFilter_EN = 1;
				PIDFilter = optarg;
				break;
			case 'O':
				OneFull_EN = 1;
				break;
			case 'h':
				usage();
				break;
			default:
				usage();
        }
    }

	//printf("annex=%c\n",annex);
    if(optind < i_argc )
        usage();

    if((0 ==Adapter_EN) || (0 == Fenum_EN))
        usage();


    char psz_tmp[128];
    sprintf(psz_tmp, "/dev/tbsmod%d/mod%d", adapter, fenum);
    if((i_frontend = open(psz_tmp, O_WRONLY)) < 0)
    {
        printf("opening device %s failed (%s)\n", psz_tmp,strerror(errno));
        exit(1);
    }

	struct FrqBndC *FBC;
 	FBC = (struct FrqBndC *)malloc(sizeof(struct FrqBndC));
	FBC->frequency = frequency;
	FBC->bandwidth = bandwidth;
	FBC->gain = gain;
	FBC->channels = channels;

	if(1 == SpiS_EN) {
		tbs_write(SPI_DEVICE, spis);//0:配置AD9789 1：FPGA升级
	}

	if(1 == Channels_EN && 1 == Enable_EN) {
		AD9789_Channel(FBC,enable);
	}

	if(1 == Channels_EN && 1 == Gain_EN) {
		AD9789_Gain(FBC);
	}

	if(1 == Frequency_EN && 1 == Bandwidth_EN && 1 == Gain_EN) {
		printf("frequency=%ld\n\n",FBC->frequency);
		AD9789_Frq(FBC);
	}

	if(1 == AD9789Init_EN)
	{
		AD9789_Configuration_Init(AD9789Init);
	}

	if(1 == ResetISDBTR_EN && 1 == Channel_EN) {
		ResetISDBT_Register(Channel);
	}
	if(1 == ReadISDBT_EN && 1 == Channel_EN && 1 == OneFull_EN) {
		ReadOneFull(Channel, ReadAdd);
	}
	if(1 == ReadISDBT_EN && 1 == Channel_EN && 0 == OneFull_EN) {
		ReadISDBT_Register(Channel, ReadAdd);
	}
	if(1 == ISDBTC_EN) {
		ISDBT_Configuration(ISDBTC);
	}

	if(1 == PIDFilter_EN) {
		PIDFilter_Configuration(PIDFilter);
	}

	if(1 == ResetCore_EN) {
		ResetCore();
	}

	return 0;
}
